﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("Stochastic MultiComponentVector Move Generator", "Generates all possible multi component moves from a given multiComponentVector.")]
  [StorableClass]
  public class StochasticMultiComponentVectorMoveGenerator : MultiComponentVectorMoveGenerator, IMultiMoveGenerator, IStochasticOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public IValueLookupParameter<IntValue> SampleSizeParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["SampleSize"]; }
    }

    public IntValue SampleSize {
      get { return SampleSizeParameter.Value; }
      set { SampleSizeParameter.Value = value; }
    }

    [StorableConstructor]
    protected StochasticMultiComponentVectorMoveGenerator(bool deserializing) : base(deserializing) { }
    protected StochasticMultiComponentVectorMoveGenerator(StochasticMultiComponentVectorMoveGenerator original, Cloner cloner) : base(original, cloner) { }
    public StochasticMultiComponentVectorMoveGenerator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator."));
      Parameters.Add(new ValueLookupParameter<IntValue>("SampleSize", "The number of moves to generate."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new StochasticMultiComponentVectorMoveGenerator(this, cloner);
    }

    public static MultiComponentVectorMove[] Apply(MultiComponentVectorEncoding multiComponentVector, IRandom random, int sampleSize) {
      MultiComponentVectorMove[] moves = new MultiComponentVectorMove[sampleSize];
      var rm = new List<MultiComponentVectorMove> (GenerateRotationMoves(multiComponentVector));
      var gm = new List<MultiComponentVectorMove>(GenerateSingleGroupingMoves(multiComponentVector));
      var sm = new List<MultiComponentVectorMove>(GenerateSwapPositionMoves(multiComponentVector));
      var cpm = new List<MultiComponentVectorMove>(GenerateChangePositionMoves(multiComponentVector));

      for (int i = 0; i < sampleSize; i++) {
        int typeDecision = random.Next(4);
        switch (typeDecision) {
          case 0: moves[i] = rm[random.Next(rm.Count)];
            break;
          case 1: moves[i] = gm[random.Next(gm.Count)];
            break;
          case 2: moves[i] = sm[random.Next(sm.Count)];
            break;
          case 3: moves[i] = cpm[random.Next(cpm.Count)];
            break;
          default:
            break;
        }
      }
      return moves;
    }

    protected override MultiComponentVectorMove[] GenerateMoves(MultiComponentVectorEncoding multiComponentVector) {
      IRandom random = RandomParameter.ActualValue;
      return Apply(multiComponentVector, random, SampleSizeParameter.ActualValue.Value);
    }
  }
}
